@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27507 $
+ * $Revision: 28153 $
  * $Author: rjongbloed $
- * $Date: 2012-04-24 02:21:12 -0500 (Tue, 24 Apr 2012) $
+ * $Date: 2012-08-06 22:45:21 -0500 (Mon, 06 Aug 2012) $
  */
 
 #ifndef PTLIB_VXML_H
@@ -272,11 +272,6 @@ class PVXMLSession : public PIndirectChannel
     virtual PBoolean TraversePrompt(PXMLElement & element);
     virtual PBoolean TraverseField(PXMLElement & element);
     virtual PBoolean TraversedField(PXMLElement & element);
-    virtual PBoolean TraverseFilled(PXMLElement & element);
-    virtual PBoolean TraverseNoInput(PXMLElement & element);
-    virtual PBoolean TraverseNoMatch(PXMLElement & element);
-    virtual PBoolean TraverseError(PXMLElement & element);
-    virtual PBoolean TraverseCatch(PXMLElement & element);
     virtual PBoolean TraverseTransfer(PXMLElement & element);
     virtual PBoolean TraversedTransfer(PXMLElement & element);
 
