@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 22375 $
+ * $Revision: 25752 $
  * $Author: rjongbloed $
- * $Date: 2009-04-06 21:47:23 -0500 (Mon, 06 Apr 2009) $
+ * $Date: 2011-05-11 21:50:32 -0500 (Wed, 11 May 2011) $
  */
 
 
@@ -51,15 +51,14 @@ class PVideoFrameInfo : public PObject
   PCLASSINFO(PVideoFrameInfo, PObject);
 
   public:
-      PVideoFrameInfo();
-
     enum ResizeMode
     {
-        eScale,
-        eCropCentre,
-        eCropTopLeft,
-        eMaxResizeMode
+      eScale,
+      eCropCentre,
+      eCropTopLeft,
+      eMaxResizeMode
     };
+    friend ostream & operator<<(ostream & strm, ResizeMode mode);
 
     enum StandardSizes {
       SQCIFWidth = 128,  SQCIFHeight = 96,
@@ -67,17 +66,40 @@ class PVideoFrameInfo : public PObject
       CIFWidth   = 352,  CIFHeight   = 288,
       CIF4Width  = 704,  CIF4Height  = 576,
       CIF16Width = 1408, CIF16Height = 1152,
-      i480Width  = 640,  i480Height  = 480,
-      p720Width  = 960,  p720Height  = 720,
+      HD480Width = 704,  HD480Height = 480,
+      i480Width  = 704,  i480Height  = 480,
+      HD720Width = 1280, HD720Height = 720,
+      p720Width  = 1280, p720Height  = 720,
+      HD1080Width= 1920, HD1080Height= 1080,
       i1080Width = 1920, i1080Height = 1080,
       HDTVWidth  = 1920, HDTVHeight  = 1080,
-      MaxWidth   = 1920, MaxHeight   = 1152
+      MaxWidth   = 1920, MaxHeight   = 1200
     };
 
+    /// Construct video frame information
+    PVideoFrameInfo();
+    PVideoFrameInfo(
+      unsigned        frameWidth,
+      unsigned        frameHeight,
+      const PString & colourFormat = "YUV420P",
+      unsigned        frameRate = 15,
+      ResizeMode      resizeMode = eScale
+    );
+
+    /** Output the contents of the object to the stream. The exact output is
+       dependent on the exact semantics of the descendent class. This is
+       primarily used by the standard <code>#operator<<</code> function.
+
+       The default behaviour is to print the class name.
+     */
+    virtual void PrintOn(
+      ostream & strm   // Stream to print the object into.
+    ) const;
+
     /**Set the frame size to be used.
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -87,7 +109,7 @@ class PVideoFrameInfo : public PObject
     /**Get the frame size being used.
 
        Default behaviour returns the value of the frameWidth and frameHeight
-       variable and returns PTrue.
+       variable and returns true.
     */
     virtual PBoolean GetFrameSize(
       unsigned & width,
@@ -106,10 +128,39 @@ class PVideoFrameInfo : public PObject
     */
     virtual unsigned GetFrameHeight() const;
 
+    /**Set the sar size to be used.
+
+       Default behaviour sets the sarWidth and sarHeight variables and
+       returns PTrue.
+    */
+    virtual PBoolean SetFrameSar(unsigned width, unsigned height);
+
+     /**Get the sar size being used.
+
+       Default behaviour returns the value of the sarWidth and sarHeight
+       variable and returns PTrue.
+    */
+    virtual PBoolean GetSarSize(
+      unsigned & width,
+      unsigned & height
+    ) const;
+
+    /** Get the width of the sar being used.
+
+        Default behaviour returns the value of the sarWidth variable
+    */
+    virtual unsigned GetSarWidth() const;
+
+    /** Get the height of the sar being used.
+
+        Default behaviour returns the value of the sarHeight variable
+    */
+    virtual unsigned GetSarHeight() const;
+    
     /**Set the video frame rate to be used on the device.
 
        Default behaviour sets the value of the frameRate variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameRate(
       unsigned rate  ///< Frames  per second
@@ -124,7 +175,7 @@ class PVideoFrameInfo : public PObject
     /**Set the colour format to be used.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue if not an empty string.
+       returns true if not an empty string.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
@@ -155,17 +206,51 @@ class PVideoFrameInfo : public PObject
       const PString & colourFormat  ///< Colour format of frame
     );
 
-    /** Parse the standard size string names ("qcif", "cif" etc)
+    /** Parse a descriptor string for the video format.
+        This is of the form [fmt ':' ] size [ '@' rate][ '/' crop ]. The size component
+        is as for the ParseSize() function.
+
+        The fmt string is the colour format such as "RGB32", "YUV420P" etc.
+
+        The rate field is a simple integer from 1 to 100.
+
+        The crop field is one of "scale", "resize" (synonym for "scale"),
+        "centre", "center", "topleft" or "crop" (synonym for "topleft").
+
+        Note no spaces are allowed in the descriptor.
       */
-    static PBoolean ParseSize(
+    bool Parse(
+      const PString & str   ///< String to parse
+    );
+
+    /** Parse the standard size string names.
+        This will parse a size desciption using either standard names: "qcif",
+        "cif", "vga", "hd1080" etc or WxY form e.g. "640x480".
+      */
+    static bool ParseSize(
       const PString & str,  ///< String to parse
       unsigned & width,     ///< Resultant width
       unsigned & height     ///< Resulatant height
     );
 
+    /**Get a width/height as a standard size string name.
+      */
+    static PString AsString(
+      unsigned width,     ///< Width to convert
+      unsigned height     ///< Height to convert
+    );
+
+    /**Get all "known" image size names.
+       Returns all standard names for sizes, e.g. "qcif", "cif", "vga",
+       "hd1080" etc.
+      */
+    static PStringArray GetSizeNames();
+
   protected:
     unsigned   frameWidth;
     unsigned   frameHeight;
+    unsigned   sarWidth;
+    unsigned   sarHeight;
     unsigned   frameRate;
     PString    colourFormat;
     ResizeMode resizeMode;
@@ -178,25 +263,25 @@ class PVideoControlInfo : public PObject
 
  public:
 
-	 typedef enum {
-        ControlPan,
-        ControlTilt,
-        ControlZoom
-     } InputControlType;
+    typedef enum {
+      ControlPan,
+      ControlTilt,
+      ControlZoom
+    } InputControlType;
 
- 	static PString AsString(const InputControlType & type);
+    static PString AsString(const InputControlType & type);
 
-	  InputControlType type;
-	  long					  min;
-	  long					  max;
-	  long					  step;
-	  long					  def;
-	  long					  flags;
-	  long					  current;
+    InputControlType type;
+    long             min;
+    long             max;
+    long             step;
+    long             def;
+    long             flags;
+    long             current;
 };
 
 
-/**This class defines a video Input device control
+/**This class defines a video Input device control (Camera controls PTZ)
 */
 
 class PVideoInputControl : public PVideoControlInfo
@@ -204,31 +289,52 @@ class PVideoInputControl : public PVideoControlInfo
     PCLASSINFO(PVideoInputControl, PVideoControlInfo);
 
 public:
-	~PVideoInputControl();
+  ~PVideoInputControl();
 
-	virtual PBoolean Pan(long value, bool absolute = false );
-	virtual PBoolean Tilt(long value, bool absolute = false);
-	virtual PBoolean Zoom(long value, bool absolute = false);
+  virtual PBoolean Pan(long value, bool absolute = false );
+  virtual PBoolean Tilt(long value, bool absolute = false);
+  virtual PBoolean Zoom(long value, bool absolute = false);
 
-	long GetPan();
-	long GetTilt();
-	long GetZoom();
+  long GetPan();
+  long GetTilt();
+  long GetZoom();
 
-	void Reset();
-	void SetCurrentPosition(const InputControlType ctype, long current);
+  void Reset();
+  void SetCurrentPosition(const InputControlType ctype, long current);
 
-	typedef std::list<PVideoControlInfo> InputDeviceControls;
+  typedef std::list<PVideoControlInfo> InputDeviceControls;
 
 protected:
-	PBoolean GetVideoControlInfo(const InputControlType ctype, PVideoControlInfo & control);
-	PBoolean GetDefaultPosition(const InputControlType ctype, long & def);
-	PBoolean GetCurrentPosition(const InputControlType ctype, long & current);
+  PBoolean GetVideoControlInfo(const InputControlType ctype, PVideoControlInfo & control);
+  PBoolean GetDefaultPosition(const InputControlType ctype, long & def);
+  PBoolean GetCurrentPosition(const InputControlType ctype, long & current);
 
-	std::list<PVideoControlInfo> m_info;
-	PMutex ccmutex;
+  std::list<PVideoControlInfo> m_info;
+  PMutex ccmutex;
 
 };
 
+/**This class defines a video Input device Interactions (Remote Inputs/Controls)
+*/
+class PVideoInteractionInfo : public PObject
+{
+  PCLASSINFO(PVideoInteractionInfo, PObject);
+
+ public:
+
+   typedef enum {
+        InteractKey,    /// Register remote KeyPresses
+        InteractMouse,    /// Register remote Mouse Movement Clicks
+        InteractNavigate,  /// Register remote Navigation commands
+    InteractRTSP,    /// Register remote RTSP (Real Time Streaming Protocol) Inputs
+    InteractOther    /// Register remote application specific Inputs
+     } InputInteractType;
+
+   static PString AsString(const InputInteractType & type);
+
+  InputInteractType type;
+};
+
 
 /**This class defines a video device.
    This class is used to abstract the few parameters that are common to both\
@@ -252,7 +358,7 @@ protected:
      "YUV410P"  YUV 4:1:0 planar
      "MJPEG"    Motion JPEG
      "UYVY422"  YUV 4:2:2 packed as U Y V Y U Y V Y ...
-     "UYV444    YUV 4:4:4 packed as U Y V   U Y V   ...
+     "UYV444"   YUV 4:4:4 packed as U Y V   U Y V   ...
                 They are used in IEEE 1394 digital cameras. The specification
                 is found at
 http://www.1394ta.org/Download/Technology/Specifications/2000/IIDC_Spec_v1_30.pdf
@@ -317,14 +423,14 @@ class PVideoDevice : public PVideoFrameInfo
       */
     virtual PBoolean OpenFull(
       const OpenArgs & args,      ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue  ///< Immediately start device
+      PBoolean startImmediate = true  ///< Immediately start device
     );
 
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     ) = 0;
 
     /**Determine if the device is currently open.
@@ -351,7 +457,7 @@ class PVideoDevice : public PVideoFrameInfo
     /**Set the video format to be used.
 
        Default behaviour sets the value of the videoFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetVideoFormat(
       VideoFormat videoFormat   ///< New video format
@@ -374,7 +480,7 @@ class PVideoDevice : public PVideoFrameInfo
        special value of -1 will find the first working channel number.
 
        Default behaviour sets the value of the channelNumber variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetChannel(
       int channelNumber  ///< New channel number for device.
@@ -397,12 +503,12 @@ class PVideoDevice : public PVideoFrameInfo
     );
 
     /**Get the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean GetVFlipState();
 
     /**Set the video conversion vertical flip state.
-       Default action is to return PFalse.
+       Default action is to return false.
      */
     virtual PBoolean SetVFlipState(
       PBoolean newVFlipState    ///< New vertical flip state
@@ -411,7 +517,7 @@ class PVideoDevice : public PVideoFrameInfo
     /**Get the minimum & maximum size of a frame on the device.
 
        Default behaviour returns the value 1 to UINT_MAX for both and returns
-       PFalse.
+       false.
     */
     virtual PBoolean GetFrameSizeLimits(
       unsigned & minWidth,   ///< Variable to receive minimum width
@@ -440,17 +546,29 @@ class PVideoDevice : public PVideoFrameInfo
     virtual PBoolean SetFrameSizeConverter(
       unsigned width,                   ///< New width of frame
       unsigned height,                  ///< New height of frame
-	  PBoolean  /*bScaleNotCrop*/           ///< Not used.
-	  )  { return SetFrameSizeConverter(width,height,eScale); }
+    PBoolean  /*bScaleNotCrop*/           ///< Not used.
+    )  { return SetFrameSizeConverter(width,height,eScale); }
 
 
+    /**Set the nearest available frame size to be used.
+
+       Note that devices may not be able to produce the requested size, so
+       this function picks the nearest available size.
+
+       Default behaviour simply calls SetFrameSize().
+    */
+    virtual PBoolean SetNearestFrameSize(
+      unsigned width,   ///< New width of frame
+      unsigned height   ///< New height of frame
+    );
+
     /**Set the frame size to be used.
 
        Note that devices may not be able to produce the requested size, and
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -460,7 +578,7 @@ class PVideoDevice : public PVideoFrameInfo
     /**Get the frame size being used.
 
        Default behaviour returns the value of the frameWidth and frameHeight
-       variable and returns PTrue.
+       variable and returns true.
     */
     virtual PBoolean GetFrameSize(
       unsigned & width,
@@ -605,7 +723,7 @@ class PVideoOutputDevice : public PVideoDevice
        If driverName is an empty string or the value "*" then this will return
        a list of unique device names across all of the available drivers. If
        two drivers have identical names for devices, then the string returned
-       will be of the form driver+'\t'+device.
+       will be of the form driver+'\\t'+device.
     */
     static PStringArray GetDriversDeviceNames(
       const PString & driverName,         ///< Name of driver
@@ -637,7 +755,7 @@ class PVideoOutputDevice : public PVideoDevice
     static PVideoOutputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = PTrue,         ///< Immediately start display
+      PBoolean startImmediate = true,         ///< Immediately start display
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
 
@@ -645,20 +763,20 @@ class PVideoOutputDevice : public PVideoDevice
     */
     static PVideoOutputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue          ///< Immediately start display
+      PBoolean startImmediate = true          ///< Immediately start display
     );
 
     /**Close the device.
       */
-    virtual PBoolean Close() { return PTrue; }
+    virtual PBoolean Close() { return true; }
 
     /**Start the video device I/O display.
       */
-    virtual PBoolean Start() { return PTrue; }
+    virtual PBoolean Start() { return true; }
 
     /**Stop the video device I/O display.
       */
-    virtual PBoolean Stop() { return PTrue; }
+    virtual PBoolean Stop() { return true; }
 
     /** Is the device a camera, and obtain video
      */
@@ -672,7 +790,7 @@ class PVideoOutputDevice : public PVideoDevice
       unsigned width,
       unsigned height,
       const BYTE * data,
-      PBoolean endFrame = PTrue
+      PBoolean endFrame = true
     ) = 0;
     virtual PBoolean SetFrameData(
       unsigned x,
@@ -683,6 +801,26 @@ class PVideoOutputDevice : public PVideoDevice
       PBoolean endFrame,
       unsigned flags
     );
+    virtual PBoolean SetFrameData(
+      unsigned x,
+      unsigned y,
+      unsigned width,
+      unsigned height,
+      unsigned sarwidth,
+      unsigned sarheight,
+      const BYTE * data,
+      PBoolean endFrame,
+      unsigned flags,
+      const void * mark
+    );
+
+    /**Allow the outputdevice decide whether the 
+        decoder should ignore decode hence not render
+        any output. 
+
+        Returns: false if to decode and render.
+      */
+    virtual PBoolean DisableDecode();
 
     /**Get the position of the output device, where relevant. For devices such as
        files, this always returns zeros. For devices such as Windows, this is the
@@ -694,6 +832,17 @@ class PVideoOutputDevice : public PVideoDevice
       int & x,  // X position of device surface
       int & y   // Y position of device surface
     ) const;
+
+    /**Set the position of the output device, where relevant. For devices such as
+       files, this does nothing. For devices such as Windows, this sets the
+       position of the window on the screen.
+       
+       Returns: TRUE if the position can be set.
+      */
+    virtual bool SetPosition(
+      int x,  // X position of device surface
+      int y   // Y position of device surface
+    );
 };
 
 
@@ -709,14 +858,14 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
     PVideoOutputDeviceRGB();
 
     /**Set the colour format to be used.
-       Note that this function does not do any conversion. If it returns PTrue
+       Note that this function does not do any conversion. If it returns true
        then the video device does the colour format in native mode.
 
        To utilise an internal converter use the SetColourFormatConverter()
        function.
 
        Default behaviour sets the value of the colourFormat variable and then
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetColourFormat(
       const PString & colourFormat // New colour format for device.
@@ -728,7 +877,7 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
        this function will fail.  See SetFrameSizeConverter().
 
        Default behaviour sets the frameWidth and frameHeight variables and
-       returns PTrue.
+       returns true.
     */
     virtual PBoolean SetFrameSize(
       unsigned width,   ///< New width of frame
@@ -750,7 +899,7 @@ class PVideoOutputDeviceRGB : public PVideoOutputDevice
       unsigned width,
       unsigned height,
       const BYTE * data,
-      PBoolean endFrame = PTrue
+      PBoolean endFrame = true
     );
 
     /**Indicate frame may be displayed.
@@ -782,7 +931,7 @@ class PVideoOutputDevicePPM : public PVideoOutputDeviceRGB
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name (filename base) to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     );
 
     /**Determine if the device is currently open.
@@ -833,7 +982,7 @@ class PVideoInputDevice : public PVideoDevice
        If driverName is an empty string or the value "*" then this will return
        a list of unique device names across all of the available drivers. If
        two drivers have identical names for devices, then the string returned
-       will be of the form driver+'\t'+device.
+       will be of the form driver+'\\t'+device.
     */
     static PStringArray GetDriversDeviceNames(
       const PString & driverName,         ///< Name of driver
@@ -868,7 +1017,7 @@ class PVideoInputDevice : public PVideoDevice
     static PVideoInputDevice *CreateOpenedDevice(
       const PString & driverName,         ///< Name of driver
       const PString & deviceName,         ///< Name of device
-      PBoolean startImmediate = PTrue,         ///< Immediately start grabbing
+      PBoolean startImmediate = true,         ///< Immediately start grabbing
       PPluginManager * pluginMgr = NULL   ///< Plug in manager, use default if NULL
     );
 
@@ -876,13 +1025,14 @@ class PVideoInputDevice : public PVideoDevice
     */
     static PVideoInputDevice *CreateOpenedDevice(
       const OpenArgs & args,              ///< Parameters to set on opened device
-      PBoolean startImmediate = PTrue          ///< Immediately start display
+      PBoolean startImmediate = true          ///< Immediately start display
     );
 
-	typedef struct {
-	   std::list<PVideoFrameInfo> framesizes;
-	   std::list<PVideoControlInfo> controls;
-	} Capabilities;
+  typedef struct {
+     std::list<PVideoFrameInfo> framesizes;
+     std::list<PVideoControlInfo> controls;
+     std::list<PVideoInteractionInfo> interactions;
+  } Capabilities;
 
     /**Retrieve a list of Device Capabilities
       */
@@ -907,15 +1057,20 @@ class PVideoInputDevice : public PVideoDevice
       PPluginManager * pluginMgr = NULL     ///< Plug in manager, use default if NULL
     );
 
+    /**Get the devices video Input controls
+        By Default return NULL;
+      */
+    virtual PVideoInputControl * GetVideoInputControls();
+
     /**Open the device given the device name.
       */
     virtual PBoolean Open(
       const PString & deviceName,   ///< Device name to open
-      PBoolean startImmediate = PTrue    ///< Immediately start device
+      PBoolean startImmediate = true    ///< Immediately start device
     ) = 0;
 
     virtual PBoolean Close(
-    ) { return PTrue; }
+    ) { return true; }
 
     /** Is the device a camera, and obtain video
      */
@@ -925,6 +1080,18 @@ class PVideoInputDevice : public PVideoDevice
       */
     virtual PBoolean IsCapturing() = 0;
 
+    /**Set the nearest available frame size to be used.
+
+       Note that devices may not be able to produce the requested size, so
+       this function picks the nearest available size.
+
+       Default behaviour simply calls SetFrameSize().
+    */
+    virtual PBoolean SetNearestFrameSize(
+      unsigned width,   ///< New width of frame
+      unsigned height   ///< New height of frame
+    );
+
     /**Grab a frame.
       */
     virtual PBoolean GetFrame(
@@ -955,9 +1122,28 @@ class PVideoInputDevice : public PVideoDevice
       PINDEX * bytesReturned = NULL  ///< Optional bytes returned.
     ) = 0;
 
-    /**Try all known video formats & see which ones are accepted by the video driver
-     */
-    virtual PBoolean TestAllFormats() = 0;
+    /**Pass data to the inputdevice for flowControl determination.
+      */
+    virtual bool FlowControl(const void * flowData);
+
+    /**Set the capture modes for implementations that support them.
+       For example with Video For Windows, this is used to select picture (0)
+       or video (1) modes.
+
+       In picture-mode the implementation requests a single frame from the
+       connected camera device. The camera device then does nothing until the
+       frame has been processed and the next is requested.
+
+       In video-mode the camera continuously sends new frames.
+
+       The default implementation does nothing but returns PFalse.
+      */
+    virtual bool SetCaptureMode(unsigned mode);
+
+    /**Returns the current capture mode. See SetCaptureMode() for more details.
+       A return value of -1 indicates an error or the mode is not supported.
+    */
+    virtual int GetCaptureMode() const;
 };
 
 
@@ -979,6 +1165,25 @@ template <class className> class PVideoInputPluginServiceDescriptor : public PDe
   static PVideoInputPluginServiceDescriptor<PVideoInputDevice_##name> PVideoInputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoInputDevice, &PVideoInputDevice_##name##_descriptor)
 
+PPLUGIN_STATIC_LOAD(FakeVideo, PVideoInputDevice);
+
+#ifdef P_APPSHARE
+  PPLUGIN_STATIC_LOAD(Application, PVideoInputDevice);
+#endif
+
+#if P_FFVDEV
+  PPLUGIN_STATIC_LOAD(FFMPEG, PVideoInputDevice);
+#endif
+
+#if P_VIDFILE
+  PPLUGIN_STATIC_LOAD(YUVFile, PVideoInputDevice);
+#endif
+
+#ifdef P_DIRECTSHOW
+  PPLUGIN_STATIC_LOAD(DirectShow, PVideoInputDevice);
+#endif
+
+
 ////////////////////////////////////////////////////////
 //
 // declare macros and structures needed for video output plugins
@@ -995,6 +1200,15 @@ template <class className> class PVideoOutputPluginServiceDescriptor : public PD
   static PVideoOutputPluginServiceDescriptor<PVideoOutputDevice_##name> PVideoOutputDevice_##name##_descriptor; \
   PCREATE_PLUGIN(name, PVideoOutputDevice, &PVideoOutputDevice_##name##_descriptor)
 
+#if _WIN32
+  PPLUGIN_STATIC_LOAD(Window, PVideoOutputDevice);
+#endif
+
+#if P_SDL
+  PPLUGIN_STATIC_LOAD(SDL, PVideoOutputDevice);
+#endif
+
+
 ////////////////////////////////////////////////////////
 //
 // declare classes needed for access to simple video font
@@ -1020,3 +1234,4 @@ class PVideoFont : public PObject
 #endif   // PTLIB_PVIDEOIO_H
 
 // End Of File ///////////////////////////////////////////////////////////////
+
